/**
  Physics simulator demo program.
  
  Dr. Orion Sky Lawlor, olawlor@acm.org, 2011-01-20 (Public Domain)
*/
#include "physics/world.h"
#include "physics/world.cpp" /* <- LAME!  include library here, for easy linking */
#include "physics/config.cpp"

/* One simulated particle */
class particle {
public:
	vec3 color; // RGB
	vec3 pos, vel; // in world coordinates-- meters
	float age; // time in seconds since created
	
	void simulate(double dt) {
	/* Newtonian motion */
		vec3 acc(0,0,-9.8); 
		pos+=dt*vel;
		vel+=dt*acc;
		
	/* Boundary conditions */
		if (pos.z<0.0) { /* hit the ground */
			vel.z=fabs(vel.z)-0.2;
			
			vec3 F=-0.1*vel;
			vec3 A=F/1.0;
			vel+=dt*A;
			
			pos.z=0.0;
		}	
		
		float r=0.3;
		vec3 center=vec3(0.0);
		if (length(pos-center)<r) { /* hit inner sphere */
			vel=0.7*reflect(vel,normalize(pos-center));
			pos=normalize(pos-center)*r+center;
		}
		r=3.0;
		center=vec3(0.0,0.0,r);
		if (length(pos-center)>r) { /* hit outer sphere */
			vel=0.7*reflect(vel,normalize(pos-center));
			pos=normalize(pos-center)*r+center;
		}
	}
};

/* Random float between 0 and 1 */
float randf(void) {
	float f=rand();
	return f/RAND_MAX;
}

/* Random vector where x y and z vary between 0 and 1 */
vec3 randVec3(void) {
	return vec3(randf(),randf(),randf());
}


/* A particle system */
class particlesystem : public physics::object {
	std::vector<particle> p;
public:
	particlesystem(void) 
		:physics::object(0.01)  /* <- our timestep, in seconds */
	{
		for (int i=0;i<10000;i++) {
			particle np;
			np.color=1.0*randVec3();
			np.pos=randVec3()+vec3(-0.5,-0.5,0.7); /* push up */
			np.vel=vec3(-1.7,0.8,5.3); // (randVec3()-vec3(0.5));
			p.push_back(np);
		}
	}
	
	void simulate(physics::library &lib) {
		float slowmo=0.3;
		for (unsigned int i=0;i<p.size();i++) {
			p[i].simulate(timestep.dt*slowmo);
		}
	}
	
	void draw(physics::library &lib) {
		glPushMatrix();
		glColor4f(0.5f,0.5f,0.5f,1.0f);
		glEnable(GL_POINT_SMOOTH);
		glPointSize(10.0);
		glBegin(GL_POINTS);
		for (unsigned int i=0;i<p.size();i++) {
			glColor3fv(p[i].color);
			glVertex3fv(p[i].pos);
		}
		glEnd();
		glPopMatrix();
	}
};


/* Called create a new simulation */
void physics_setup(physics::library &lib) {
	lib.world->add(new particlesystem());
}

/* Called every frame to draw everything */
void physics_draw_frame(physics::library &lib) {
	/* Point and line smoothing needs normal alpha blending */
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	
	/* No depth while drawing grid */
	glDisable(GL_DEPTH_TEST);
	glColor3f(0.2,0.2,0.2);
	glEnable(GL_LINE_SMOOTH);
	glLineWidth(0.2);
	if (camera.z>=0.0) { /* no grid if you're underneath ground */
		glBegin(GL_LINES);
		for (int grid=-100;grid<=+100;grid++) {
			glVertex3i(-100,grid,0); glVertex3i(+100,grid,0); 
			glVertex3i(grid,-100,0); glVertex3i(grid,+100,0); 
		}
		glEnd();
	}
	
	/* Do depth while drawing objects */
	glEnable(GL_DEPTH_TEST);
	lib.world->draw(lib);
}


